/* ******************************************************************

   class    : AboutDlg
   date     : 10.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Dialogs;

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;

import CustCtrl.BaseDialog;
import CustCtrl.InsetsPanel;
import CustCtrl.ImageCanvas;
import CustCtrl.LabelArea;
import Utils.ResourceUtil;

/**
 * A dialog that displays application informations.
 *
 * @author puehringer edgar
 */
public class AboutDlg extends BaseDialog implements ActionListener {

  // class variable declarations
  private final static String TITLE             = "title";
  private final static String BUTTONS           = "buttons";
  private final static String TEXT              = "text";
  private final static String SYSINFODLG_PREFIX = "sysInfoDlg.";

  // instance variable declarations
  private Frame parent;
  private ResourceBundle resources;

  // constructors
  /** Create a new AboutDlg object. */
  public AboutDlg(Frame parent, ResourceBundle resources,
                    String resourcePrefix) {
    super(parent,true);
    this.parent    = parent;
    this.resources = resources;

    Panel panel = getContentPanel();
    Image image;
    LabelArea label;
    String str;

    setTitle(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + TITLE));
    createButtons(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + BUTTONS));
    addActionListener(this);

    GridBagLayout gridbag          = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    panel.setLayout(gridbag);
    constraints.insets     = new Insets(10,6,0,0);
    constraints.weightx    = 1.0;
    constraints.weighty    = 1.0;
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    constraints.gridheight = 1;
    image = parent.getIconImage();
    if (image != null) {
      ImageCanvas canvas = new ImageCanvas(image);

      gridbag.setConstraints(canvas,constraints);
      panel.add(canvas);
    } // end if
    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    label = new LabelArea(ResourceUtil.getResStringOrKey(resources,
                            resourcePrefix + TEXT));
    gridbag.setConstraints(label,constraints);
    panel.add(label);
    pack();
    center(parent);
    setResizable(false);
  } // end constructor AboutDlg

  // class methods
    /** Only for debuging purposes. */
  public static void main(String[] args) throws java.util.MissingResourceException {
    Frame frame = new Frame();
    AboutDlg dlg;
    ResourceBundle res;

    if (args.length < 1) {
      System.out.println("Resource file must be specified.");
      System.exit(1);
    } // end if
    res = ResourceBundle.getBundle(args[0],java.util.Locale.getDefault());
    dlg = new AboutDlg(frame,res,"aboutDlg.");
    dlg.show();
    System.out.println("isCanceled returned " + dlg.isCanceled());
    System.exit(0);
  } // end main

  // methods
  /**
   * Called to close this dialog.
   */
  protected void closeDialog(AWTEvent event) {
    if ((event instanceof WindowEvent) || !isCanceled()) {
      super.closeDialog(event);
    } // end if
  } // end closeDialog

  public void actionPerformed(ActionEvent e) {
    String cmd   = e.getActionCommand();
    String label;

    if (buttonVector.size() < 2) return;
    label = ((Button)buttonVector.elementAt(1)).getLabel();
    if (cmd.equals(label)) { // if it's the sysInfo button
      SysInfoDlg dlg = new SysInfoDlg(parent,resources,
                                      SYSINFODLG_PREFIX);

      dlg.show();
    } // end if
  } // end actionPerformed

} // end AboutDlg

/* ******************************************************************
   end of file
*********************************************************************/
