/* ******************************************************************

   compiler   : javac (jdk 1.2)
   class      : ProjectPanel
   date       : 13.07.98
   modified   : 3.5.99 - InsFileAction added on Codepad creation.

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.URL;

import javax.swing.*;
import javax.swing.text.TextAction;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import Utils.ResourceUtil;
import Utils.FrameUtil;
import Utils.StringUtil;
import Dialogs.AboutDlg;
import Dialogs.ParamDlg;
import Messages.*;

/**
 * A panel for project managment purposes.
 *
 * @author Puehringer Edgar
 */
public class ProjectPanel extends JPanel implements ListSelectionListener,
  AbstractEditor, FrameContentManager, ActionListener {

  // class variable declarations
  /** The key to the frame title resource */
  private static final String FRAME_TITLE = "Title";
  /** The key to the frame icon resource */
  private static final String FRAME_ICON = "IconImage";
  /** The key to the menu editor frame title resource */
  private static final String MEFRAME_TITLE = "meTitle";
  /** The key to the menu editor frame icon resource */
  private static final String MEFRAME_ICON = "meIconImage";
  /** Key to the status line resource */
  private static final String STATUS_LINE = "pStatusLine";
  /** The help topic to get help for this panel. */
  private static final String HELP_TOPIC = "project";
  /** Key to the class browser class resource */
  private static final String BROWSER_CLASS = "classBrowserClass";
  /** Key to the compiler class resource */
  private static final String COMPILER_CLASS = "compilerClass";
  /** Key to the linker class resource */
  private static final String LINKER_CLASS = "linkerClass";
  /** Key to the execute class resource */
  private static final String EXECUTE_CLASS = "executeClass";
  /** Key to the preprocessor class resource */
  private static final String PREPROC_CLASS = "preprocessorClass";
  /** Key to the menubar resource */
  public static final String MENUBAR = "pMenubar";
  /** Key to the toolbar resource */
  public static final String TOOLBAR = "pToolbar";
  /** Identifier of the new menu editor window action */
  public static final String MENUEDITOR_ACTION = "new-menuedit-window";
  /** Identifier of the add action */
  public static final String ADD_ACTION = "add";
  /** Identifier of the add action */
  public static final String SETDIR_ACTION = "set-dir";
  /** Identifier of the split window action */
  public static final String SPLIT_ACTION  = "split-window";
  /** Identifier of the set cmd line parameters action */
  public static final String PARAM_ACTION = "cmd-param";
  /** Identifier of the auto-save on compile action */
  public static final String ASOC_ACTION  = "save-on-compile";
  /** Identifier of the compiler options action */
  public static final String COMPILEROPTIONS_ACTION = "compiler-options";
  /** Identifier of the linker options action */
  public static final String LINKEROPTIONS_ACTION = "linker-options";
  /** Identifier of the execute options action */
  public static final String EXECUTEOPTIONS_ACTION = "exec-options";
  /** Identifier of the help options action */
  public static final String HELPOPTIONS_ACTION = "help-options";
  /** Identifier of the help content action */
  public static final String CONTENT_ACTION = "content";
  /** Identifier of the about action */
  public static final String ABOUT_ACTION = "about";

  /** Key to add directory confirm dialog text resource */
  private static final String ADDDIR1_TEXT = "addDirectoryText1";
  /** Key to add directory confirm dialog text resource */
  private static final String ADDDIR2_TEXT = "addDirectoryText2";
  /** Key to the project open error message resource */
  private static final String OPENERR_TEXT = "pOpenErrorText";
  /** Key to the new path message part 1 resource */
  private static final String NEWPATH1_TEXT = "pNewPathText1";
  /** Key to the new path message part 2 resource */
  private static final String NEWPATH2_TEXT = "pNewPathText2";
  /** Key to the new path message part 3 resource */
  private static final String NEWPATH3_TEXT = "pNewPathText3";
  /** Key to the desktop split window property */
  private static final String SPLIT_PROP = "splitWindow";

  private static final String ABOUTDLG_PREFIX = "pAboutDlg.";

  private static ResourceBundle resources;
  private static Properties desktop;
  private static FrameManager frameManager;
  private static AbstractClassBrowser classBrowser;
  private static AbstractCompiler compiler;
  private static AbstractLinker linker;
  private static AbstractExecute execute;
  private static AbstractMacroprocessor preproc;
  private static ToolLoader toolLoader;

  // instance variable declarations
  private Frame frame;
  private String originalTitle;
  private Properties project;
  private FrameManager fm;
  private Action[] externalActions;
  private EditorActionsSet editorActions;
  private Hashtable commands;
  private JMenuBar menubar;
  private JToolBar toolbar;
  private JPanel upperPanel;
  private ProjectFilePanel projectFilePanel;
  private FileManagerPanel fileManagerPanel;
  private boolean fileListSelected = false;
  private ErrorMsgPanel errorMsgPanel;
  private Statusbar status;
  /** JSplitPane being shown to the user. */
  private JSplitPane hSplit;
  private String[] projectExt;
  /** The current maximum value of the status line progress bar. */
  private int maxProgress = -1;
  /** The priority of the current progress bar controling task. */
  private int progressPrio = Message.LOW;
  /** Actions defined by the ProjectPanel class. */
  private NewWindowAction newWindowAction     = new NewWindowAction();
  private SplitWinAction splitWinAction       = new SplitWinAction();
  private AddAction addAction                 = new AddAction();
  private SetDirAction setDirAction           = new SetDirAction();
  private MakeAction makeAction               = new MakeAction(null,null,null);
  private BuildAction buildAction             = new BuildAction(null,null,null);
  private RunAction runAction                 = new RunAction(null,null,null,null);
  private SelRunAction selRunAction           = new SelRunAction(null,null,null,null);
  private SaveOnCompileAction saveOnComAction = new SaveOnCompileAction();
  private CompilerOptionsAction compOptAction = new CompilerOptionsAction();
  private LinkerOptionsAction linkOptAction   = new LinkerOptionsAction();
  private ExecuteOptionsAction execOptAction  = new ExecuteOptionsAction();
  private Action[] defaultActions = {
    newWindowAction,
    new MenuEditorAction(),
    splitWinAction,
    addAction,
    setDirAction,
    makeAction,
    buildAction,
    runAction,
    selRunAction,
    new ParamAction(),
    saveOnComAction,
    compOptAction,
    linkOptAction,
    execOptAction,
    new HelpOptionsAction(),
    new ContentAction(),
    new AboutAction()
  };

  // constructors
  public ProjectPanel(Frame frame, ResourceBundle resources,
                      Properties desktop, FrameManager fm) {
    this(frame,resources,desktop,fm,null);
  } // end constructor ProjectPanel

  public ProjectPanel(Frame frame, ResourceBundle resources,
                      Properties desktop, FrameManager fm,
                      Action[] externalActions) {
    super(true);
    this.frame           = frame;
    originalTitle        = frame.getTitle();
    this.resources       = resources;
    this.desktop         = desktop;
    this.fm              = fm;
    frameManager         = this.fm;
    this.externalActions = externalActions;
    Action[] actions;
    CompileAction compileAction;

    // load the default project properties
    project = new Properties();
    copyDefaultProject(resources,project);
    setLayout(new BorderLayout());
    // create the embedded Components
    fileManagerPanel = new FileManagerPanel(resources,project,addAction);
    fileManagerPanel.setEnabled(false);
    setDirAction.setEnabled(false);
    fileManagerPanel.addListSelectionListener(this);
    compileAction    = new CompileAction(null,null);
    actions          = new Action[1];
    actions[0]       = compileAction;
    projectFilePanel = new ProjectFilePanel(frame,resources,project,newWindowAction,
                                            actions);
    errorMsgPanel = new ErrorMsgPanel(this.frame,resources,
                                      project,newWindowAction);
    // create the upper panel
    upperPanel = new JPanel(new BorderLayout());
    upperPanel.add(projectFilePanel);
    // load the external tools
    if (toolLoader == null) {
      toolLoader = new ToolLoader(resources,project);
      toolLoader.setParentFrame(frame);
      toolLoader.addObserver(this);
      toolLoader.addObserver(errorMsgPanel);
    } // end if
    // install the command table
    editorActions = new EditorActionsSet(this);
    commands = new Hashtable();
    actions  = getActions();
    for (int i = 0; i < actions.length; i++) {
            Action a = actions[i];
            commands.put(a.getValue(Action.NAME), a);
    } // end for

    Hashtable menuItems    = new Hashtable();
    JMenuLoader menuLoader = new JMenuLoader(this.resources);

    menuLoader.setCommands(commands);
    menubar = menuLoader.createMenuBar(MENUBAR);
    add("North",menubar);

    JPanel panel = new JPanel();

    panel.setLayout(new BorderLayout());

    JToolBarLoader toolbarLoader = new SmartToolBarLoader(this.resources);

    toolbarLoader.setCommands(commands);
    toolbar = toolbarLoader.createToolBar(TOOLBAR);
    panel.add("North",toolbar);

    JSplitPane vSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                       upperPanel,
                                       errorMsgPanel);

    panel.add("Center",vSplit);
    add("Center",panel);
    status = new Statusbar(ResourceUtil.getResStringOrKey(resources,STATUS_LINE));
    add("South",status);
    // split window, if stored in desktop properties
    String str = desktop.getProperty(SPLIT_PROP);

    if ((str == null) || Boolean.valueOf(str).booleanValue()) {
      setFileManagerVisible(true);
      splitWinAction.putValue("checked",Boolean.TRUE);
    } // end if
    // enable autosave on compile, if stored in desktop properties
    str = desktop.getProperty(Const.ASOC_PROP);
    if (str == null) desktop.put(Const.ASOC_PROP,"true");
    if ((str == null) || Boolean.valueOf(str).booleanValue())
      saveOnComAction.putValue("checked",Boolean.TRUE);
    // end if
    // initalize the filename filter
    str = project.getProperty(Const.PROJECT_EXT);
    if (str != null) projectExt = ResourceUtil.splitToWords(str);
      else System.err.println("ProjectPanel: " + Const.PROJECT_EXT +
                              " not in resource file");
    // load the class browser class
    str = ResourceUtil.getResourceString(resources,BROWSER_CLASS);
    classBrowser = AbstractClassBrowser.getInstance(str,resources,project);
    if (classBrowser != null) classBrowser.addObserver(this);
    // load the compiler class
    str = ResourceUtil.getResourceString(resources,COMPILER_CLASS);
    compiler = AbstractCompiler.getInstance(str,resources,project);
    if (compiler != null) {
      compiler.addObserver(this);
      compiler.addObserver(errorMsgPanel);
      compileAction.setFrame(frame);
      compileAction.setCompiler(compiler);
      compileAction.setActive(false);
      makeAction.setFrame(frame);
      makeAction.setCompiler(compiler);
      buildAction.setFrame(frame);
      buildAction.setCompiler(compiler);
      runAction.setFrame(frame);
      runAction.setCompiler(compiler);
      selRunAction.setFrame(frame);
      selRunAction.setCompiler(compiler);
    } // end if
    compOptAction.setEnabled((compiler != null) &&
                             compiler.hasSettings());
    // load the linker plugin class
    str = ResourceUtil.getResourceString(resources,LINKER_CLASS);
    linker = AbstractLinker.getInstance(str,resources,project);
    if (linker != null) {
      linker.addObserver(this);
      linker.addObserver(errorMsgPanel);
      makeAction.setLinker(linker);
      buildAction.setLinker(linker);
      runAction.setLinker(linker);
      selRunAction.setLinker(linker);
    } // end if
    linkOptAction.setEnabled((linker != null) &&
                             linker.hasSettings());
    // load the execute plugin class
    str = ResourceUtil.getResourceString(resources,EXECUTE_CLASS);
    execute = AbstractExecute.getInstance(str,resources,project);
    if (execute != null) {
      execute.addObserver(this);
      execute.addObserver(errorMsgPanel);
      runAction.setExecute(execute);
      selRunAction.setExecute(execute);
    } // end if
    execOptAction.setEnabled((execute != null) &&
                              execute.hasSettings());
    // load the preprocessor class
    str = ResourceUtil.getResourceString(resources,PREPROC_CLASS);
    preproc = AbstractMacroprocessor.getInstance(str,resources,project);
    if (preproc != null) {
      preproc.addObserver(this);
      preproc.addObserver(errorMsgPanel);
    } // end if
    // Add a window adapter that performs cleanup activities
    frame.addWindowListener(new WindowAdapter() {

        public void windowClosed(WindowEvent e) {
          AbstractHelpWindow.quit();
          ProjectPanel.saveDesktopProperties();
          System.exit(0);
        } // end windowClosed

      } // end anonymous class
    );
  } // end constructor ProjectPanel

  // class methods
  /** Return a handle to the class browser. */
  public static AbstractClassBrowser getClassBrowser() {
    return classBrowser;
  } // end getClassBrowser

  /** Return a handle to the source code template preprocessor. */
  public static AbstractMacroprocessor getPreprocessor() {
    return preproc;
  } // end getPreprocessor

  /** Save the desktop settings to a file specifed in the resource bundle. */
  public static boolean saveDesktopProperties() {
    boolean res = true;
    String name = ResourceUtil.getResourceString(resources,Const.DESKTOP_NAME);
    File file   = new File(name);

    try {
      OutputStream fos         = new FileOutputStream(file);
      BufferedOutputStream out = new BufferedOutputStream(fos);

      desktop.store(out,file.getAbsolutePath());
      out.close();
      fos.close();
    } catch (IOException excep) {
      excep.printStackTrace();
      res = false;
    }
    return res;
  } // end saveDesktopProperties

  /** Return a handle to the frame manager. */
  public static FrameManager getFrameManager() {
    return frameManager;
  } // end getFrameManager

  // methods
  /**
   * Fetch the list of actions supported by this panel.
   */
  public Action[] getActions() {
    if (externalActions == null)
      return TextAction.augmentList(fileManagerPanel.getActions(),
               TextAction.augmentList(projectFilePanel.getActions(),
                 TextAction.augmentList(editorActions.getActions(),
                   TextAction.augmentList(defaultActions,
                                          toolLoader.getActions()))));
    else
      return TextAction.augmentList(fileManagerPanel.getActions(),
               TextAction.augmentList(projectFilePanel.getActions(),
                 TextAction.augmentList(editorActions.getActions(),
                   TextAction.augmentList(defaultActions,
                     TextAction.augmentList(externalActions,
                                            toolLoader.getActions())))));
    // end if
  } // end getActions

  /**
   * Turn the file manager on and off.
   */
  protected void setFileManagerVisible(boolean b) {
    if (b == (hSplit != null)) return;
    if (b) {
      Dimension d = upperPanel.getSize();

      fileManagerPanel.setEnabled(true);
      setDirAction.setEnabled(true);
      upperPanel.remove(projectFilePanel);
      d.width /= 10;
      fileManagerPanel.setMinimumSize(d);
      projectFilePanel.setMinimumSize(d);
      hSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,fileManagerPanel,
                              projectFilePanel);
      hSplit.setContinuousLayout(false);
      upperPanel.add(hSplit,BorderLayout.CENTER);
      validate();
      upperPanel.repaint();
    }
    else {
      upperPanel.remove(hSplit);
      hSplit.remove(projectFilePanel);
      hSplit = null;
      projectFilePanel.setMinimumSize(null);
      upperPanel.add(projectFilePanel,BorderLayout.CENTER);
      fileManagerPanel.setEnabled(false);
      setDirAction.setEnabled(false);
      upperPanel.validate();
      upperPanel.repaint();
      System.gc();
    } // end if
  } // end setFileManagerVisible

  /**
   * Adds all source files of a directory to the project file
   * pane, if the user wants this.
   *
   * @return False if operation was canceled, true otherwise.
   */
  protected boolean addDirectory(File f) {
    FilenameFilter filter = fileManagerPanel;
    String absPath        = fileManagerPanel.getPath() + File.separator;
    String text1,text2,path;
    int numFiles = f.list(filter).length;
    int result;

    if (numFiles > 0) {
      text1 = ResourceUtil.getResStringOrKey(resources,ADDDIR1_TEXT);
      path  = f.getPath();
      if (path.startsWith(absPath)) path = path.substring(absPath.length());
      text2 = ResourceUtil.getResStringOrKey(resources,ADDDIR2_TEXT);
      result = JOptionPane.showConfirmDialog(frame,
                 text1 + " \"" + path + "\" " + text2,
                 originalTitle,
                 JOptionPane.YES_NO_CANCEL_OPTION,
                 JOptionPane.QUESTION_MESSAGE);
    }
    else {
      result = JOptionPane.YES_OPTION;
    } // end if
    switch (result) {
      case JOptionPane.YES_OPTION: {
        Vector vector      = new Vector();
        String[] files     = f.list();
        boolean terminated = false;

        for (int i=0; i<files.length; i++) {
          File file = new File(f,files[i]);
          if (file.isDirectory()) {
            if (!addDirectory(file)) {
              terminated = true;
              break;
            } // end if
          }
          else {
            if (filter.accept(f,files[i])) vector.addElement(file);
          } // end if
        } // end for
        switch (vector.size()) {
          case 0: break;
          case 1:
            if (!projectFilePanel.addFile((File)vector.firstElement()))
              Toolkit.getDefaultToolkit().beep();
          break;
          default: {
            int size = vector.size();
            File[] aFiles = new File[size];

            for (int i=0; i<size; i++)
              aFiles[i] = (File)vector.elementAt(i);
            if (!projectFilePanel.addFiles(aFiles))
              Toolkit.getDefaultToolkit().beep();
          }
        } // end switch
        if (terminated) return false;
      } break;
      case JOptionPane.NO_OPTION:
      break;
      default: // CANCEL_OPTION
        return false;
    } // end switch
    return true;
  } // end addDirectory

  /** Invoke a scan of the class browser. */
  protected void updateClassBrowser() {
    if (classBrowser != null) classBrowser.update(frame);
  } // end updateClassBrowser

  private void copyDefaultProject(ResourceBundle res, Properties proj) {
    String key    = ResourceUtil.getResourceString(res,Const.COPY_ON_LOAD);
    String[] keys = ResourceUtil.splitToWords(key);

    for (int i=0; i<keys.length; i++) {
      String str = desktop.getProperty(keys[i]);
      if (str == null) str = ResourceUtil.getResourceString(res,keys[i]);
      if (str == null) {
        System.err.println("ProjectPanel: missing resource: " + keys[i]);
        continue;
      } // end if
      proj.put(keys[i],str);
    } // end for
    proj.put(Const.PROJECT_DIR,
             ResourceUtil.fileToURL(System.getProperty("user.dir")));
  } // end copyDefaultProject

  //--------- ListSelectionListener ---------------------------------
  public void valueChanged(ListSelectionEvent e) {
    JList l = (JList)e.getSource();

    if (fileListSelected != l.isSelectionEmpty()) return;
    fileListSelected = !l.isSelectionEmpty();
    addAction.setEnabled(fileListSelected);
  } // end valueChanged

  //--------- AbstractEditor ----------------------------------------
  /** Returns the hosting frame. */
  public Frame getFrame() {
    return frame;
  } // end getFrame

  /** Returns the resource bundle. */
  public ResourceBundle getResources() {
    return resources;
  } // end getResources

  /** Returns properties containing desktop settings. */
  public Properties getDesktopProperties() {
    return desktop;
  } // end getDesktopProperties

  /** Save the document to the spezified file. */
  public boolean save(File file) throws IOException {
    OutputStream fos         = new FileOutputStream(file);
    BufferedOutputStream out = new BufferedOutputStream(fos);

    project.put(Const.PROJECTFILE_PATH,
                ResourceUtil.fileToURL(file.getAbsolutePath()));
    project.store(out,file.getAbsolutePath());
    out.close();
    fos.close();
    project.put(Const.PROJECT_CHANGED,""+false);
    getFrameManager().setTitle(frame,file.getName() + " - " + originalTitle);
    updateClassBrowser();
    return true;
  } // end save

  /** Load the document from the spezified file. */
  public boolean load(File file) throws IOException {
    FileInputStream fis    = new FileInputStream(file);
    BufferedInputStream in = new BufferedInputStream(fis);
    Properties tmp         = new Properties();
    Enumeration enum;
    String magic;
    Object obj;
    String newPath;
    int res;
    boolean changed = false;

    tmp.load(in);
    in.close();
    fis.close();
    magic = ResourceUtil.getResourceString(resources,Const.MAGIC_KEY);
    if (!magic.equals(tmp.getProperty(Const.MAGIC_KEY))) {
      String[] extensions = getExtensions();

      res = JOptionPane.CANCEL_OPTION;
      for (int i=0; i<extensions.length; i++)
        if (file.getName().endsWith(extensions[i])) res = JOptionPane.OK_OPTION;
      //end for
      if (res != JOptionPane.OK_OPTION)
        res = JOptionPane.showConfirmDialog(frame,"\"" + file.getPath() + "\" " +
                ResourceUtil.getResStringOrKey(resources,OPENERR_TEXT),
                originalTitle,JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.ERROR_MESSAGE);
      // end if
      if (res == JOptionPane.OK_OPTION) {
        ActionEvent ae = new ActionEvent(this,ActionEvent.ACTION_PERFORMED,
                                         file.getAbsolutePath());

        newWindowAction.actionPerformed(ae);
      } // end if
      return false;
    } // end if
    newPath = getNewPath(tmp,file);
    if (newPath != null) {
      String projPath = ResourceUtil.urlToFile(tmp.getProperty(Const.PROJECT_DIR));

      res = JOptionPane.showConfirmDialog(frame,
              ResourceUtil.getResStringOrKey(resources,NEWPATH1_TEXT) +
              "\"" + projPath + "\"" +
              ResourceUtil.getResStringOrKey(resources,NEWPATH2_TEXT) +
              "\"" + newPath + "\"" +
              ResourceUtil.getResStringOrKey(resources,NEWPATH3_TEXT),
              originalTitle,JOptionPane.YES_NO_CANCEL_OPTION,
              JOptionPane.QUESTION_MESSAGE);
      switch(res) {
        case JOptionPane.YES_OPTION:
          replacePath(tmp,projPath,newPath);
          changed = true;
        break;
        case JOptionPane.NO_OPTION:  /* do nothing */ break;
        default:                     return false; /* CANCEL */
      } // end switch
    } // end if
    project.clear();
    enum = tmp.keys();
    while (enum.hasMoreElements()) {
      obj = enum.nextElement();
      project.put(obj,tmp.get(obj));
    } // end while
    project.put(Const.PROJECTFILE_PATH,
                ResourceUtil.fileToURL(file.getAbsolutePath()));
    project.put(Const.PROJECT_CHANGED,"" + changed);
    getFrameManager().setTitle(frame,file.getName() + " - " + originalTitle);
    projectFilePanel.updateList();
    updateClassBrowser();
    return true;
  } // end load

  /** Returns the new project directory, if the proj. file was moved. */
  private final String getNewPath(Properties prop, File file)
    throws FileNotFoundException {
    String oldPath  = prop.getProperty(Const.PROJECTFILE_PATH);
    String newPath  = file.getParent();
    String projPath,relPath;

    oldPath = ResourceUtil.urlToFile(oldPath);
    oldPath = (new File(oldPath)).getParent();
    if (oldPath == null) oldPath = "";
    if (newPath == null) newPath = "";
    if (newPath.equals(oldPath)) return null;
    projPath = prop.getProperty(Const.PROJECT_DIR);
    projPath = ResourceUtil.urlToFile(projPath);
    relPath  = StringUtil.relativePath(oldPath,projPath);
    if (relPath.equals("")) return newPath;
    if (!newPath.endsWith(relPath))
      throw new FileNotFoundException(relPath + " not found.");
    // end if
    newPath = newPath.substring(0,newPath.length() - relPath.length());
    if ((newPath.length() > 1) && newPath.endsWith(File.separator))
      newPath = newPath.substring(0,newPath.length() - File.separator.length());
    // end if
    return newPath;
  } // end getNewPath

  /** Returns the new project directory, if the proj. file was moved. */
  private final void replacePath(Properties prop, String oldPath,
                                 String newPath) {
    Enumeration enum = prop.keys();

    oldPath = ResourceUtil.fileToURL(oldPath);
    newPath = ResourceUtil.fileToURL(newPath);
    while (enum.hasMoreElements()) {
      String key   = (String)enum.nextElement();
      String value = prop.getProperty(key);

      if (value.startsWith(oldPath)) {
        value = newPath + value.substring(oldPath.length());
        prop.put(key,value);
      } // end if
    } // end while
  } // end replacePath

  /** Returns an array containing all extensions of source files. */
  private final String[] getExtensions() {
    String str;

    str = project.getProperty(Const.SOURCE_EXT);
    /* DEBUG */ Utils.Assert.assert(Utils.Assert.ENABLED && (str != null),"Extension for source files not found");
    return ResourceUtil.splitToWords(str);
  } // end getExtensions


  /** Create a new empty document. */
  public boolean create(boolean alternate) {
    copyDefaultProject(resources,project);
    project.remove(Const.PROJECTFILE_PATH);
    project.put(Const.PROJECT_CHANGED,"" + false);
    getFrameManager().setTitle(frame,originalTitle);
    projectFilePanel.updateList();
    updateClassBrowser();
    return true;
  } // end create

  /** Returns true, if the document has been changed. */
  public boolean isChanged() {
    String str = project.getProperty(Const.PROJECT_CHANGED);

    return (str != null) && Boolean.valueOf(str).booleanValue();
  } // end isChanged

  /** Return a default file to display in the save file dialog. */
  public File getDefaultFile() {
    File file = editorActions.getCurrentFile();

    if (file != null) return file;
    if ((projectExt == null) || (projectExt.length < 1)) return null;
    return new File("*" + projectExt[0]);
  } // end getDefaultFile

  public boolean accept(File dir, String name) {
    if (projectExt == null) return true;
    for (int i=0; i<projectExt.length; i++)
      if (name.endsWith(projectExt[i])) return true;
    return false;
  } // end accept

  //--------- FrameContentManager -----------------------------------
  /** Returns true if the frame manager can close this frame. */
  public boolean canClose() {
    return editorActions.canClose();
  } // end canClose

  /** Returns true, because this is an application main frame. */
  public boolean isMainFrame() {
    return true;
  } // end isMainFrame

  /**
   * Notified if the class browser scans or if the FrameManager
   * window list changes.
   */
  public synchronized void update(Observable o, Object arg) {
    if (arg instanceof ProgressMessage) {
      ProgressMessage msg = (ProgressMessage)arg;

      // do nothing if a message with low priority tries to interrupt
      // a message with high priority.
      if (msg.getPriority() < progressPrio) return;

      if (msg instanceof ClearProgressMessage) {
        status.setLabel(ResourceUtil.getResStringOrKey(resources,STATUS_LINE));
        status.removeProgress();
        progressPrio = Message.LOW;
      }
      else {
        status.setLabel(msg.getDescription());
        if (maxProgress != msg.max) {
          maxProgress = msg.max;
          status.setProgress(msg.cur,maxProgress);
        }
        else {
          status.setProgress(msg.cur);
        } // end if
        progressPrio = msg.getPriority();
      } // end if
    } else if (arg instanceof LockMessage) {
      setCompileActionsEnabled(false);
    } else if (arg instanceof UnlockMessage) {
      setCompileActionsEnabled(true);
    } else if (arg instanceof AddFileMessage) {
      File f = ((AddFileMessage)arg).getFile();

      if (!projectFilePanel.addFile(f)) getToolkit().beep();
    } else if (arg instanceof DeleteFileMessage) {
      File f = ((DeleteFileMessage)arg).getFile();

      if (!projectFilePanel.removeFile(f)) getToolkit().beep();
    } else if (arg instanceof Enumeration) {
      Enumeration enum = (Enumeration)arg;
      int index        = menubar.getMenuCount() - 2;
      int i            = 0;
      JMenu menu;

      if (index < 0) {
        getToolkit().beep();
        return;
      } // end if
      menu = menubar.getMenu(index);
      menu.removeAll();
      if (!enum.hasMoreElements())
        enum = getFrameManager().getFrames();
      // end if
      while (enum.hasMoreElements()) {
        Frame f = (Frame)enum.nextElement();
        JMenuItem mi;

        if (f == frame) mi = new JCheckBoxMenuItem(f.getTitle(),true);
          else mi = new JMenuItem(f.getTitle());
        mi.setActionCommand("" + (i++));
        mi.addActionListener(this);
        menu.add(mi);
      } // end while
    } // end if
  } // end update

  private final void setCompileActionsEnabled(boolean enbl) {
    Enumeration enum = fm.getFrames();
    Frame frame;
    Object obj;
    Action[] actions;

    while (enum.hasMoreElements()) {
      frame = (Frame)enum.nextElement();
      obj   = FrameUtil.contains(frame,CodepadPanel.class);
      if (obj == null) {
        obj     = FrameUtil.contains(frame,ProjectPanel.class);
        actions = ((ProjectPanel)obj).getActions();
        if (obj == null) continue;
      }
      else {
        actions = ((CodepadPanel)obj).getActions();
      } // end if
      for (int i=0; i<actions.length; i++) {
        if (actions[i] instanceof CompileAction) {
          actions[i].setEnabled(enbl);
        } // end if
      } // end for
    } // end while
  } // end setCompileActionsEnabled

  // --- ActionListener ---------------------------------------------
  /** Invoked when an action occurs. */
  public void actionPerformed(ActionEvent e) {
    Object obj = e.getSource();

    if (obj instanceof JCheckBoxMenuItem) {
      ((JCheckBoxMenuItem)obj).setState(true);
    }
    else {
      int index        = Integer.parseInt(e.getActionCommand());
      int i            = 0;
      Enumeration enum = getFrameManager().getFrames();

      while (enum.hasMoreElements()) {
        Frame f = (Frame)enum.nextElement();

        if ((i++) == index) {
          f.toFront();
          f.requestFocus();
          return;
        } // end if
      } // end while
    } // end if
  } // end actionPerformed

  // inner classes
  /** Constructs a new editor window. */
  class NewWindowAction extends AbstractAction {

    // constructors
    NewWindowAction() {
      super(Const.NEWWINDOW_ACTION);
    } // end constructor NewWindowAction

    // methods
    public void actionPerformed(ActionEvent e) {
      Object source    = e.getSource();
      String cmd       = e.getActionCommand();
      boolean loadAble = (((source instanceof ProjectFilePanel) ||
                           (source instanceof ProjectPanel) ||
                           (source instanceof ErrorMsgPanel) ||
                           (source instanceof AbstractClassBrowser)) &&
                          (cmd != null));
      File file        = null;
      int lineNr       = -1;
      Frame theFrame;

      // The command part of the action event can contain a file which
      // will be loaded on creating the new window. Optionally, there
      // can be a line number followed by a blank as prefix.
      if (loadAble) {
        String[] aStr = ResourceUtil.splitToWords(cmd);

        if (aStr.length > 1) try {
          lineNr = Integer.valueOf(aStr[0]).intValue();
          cmd    = cmd.substring(aStr[0].length() + 1);
        } catch (NumberFormatException excep) {}
        file = new File(cmd);
        if (file.exists()) {
          theFrame = getFrameContaining(file);
          if (theFrame != null) {
            theFrame.toFront();
            theFrame.requestFocus();
            if (lineNr >= 0) {
              CodepadPanel cpp;

              cpp = (CodepadPanel)FrameUtil.contains(theFrame,CodepadPanel.class);
              if (cpp != null) {
                cpp.goToLine(lineNr);
                cpp.getEditor().requestFocus();
              } // end if
            } // end if
            return;
          } // end if
        }
        else {
          file = null;
        } // end if
      } // end if

      String title      = ResourceUtil.getResStringOrKey(resources,FRAME_TITLE);
      JFrame frame      = new JFrame(title);
      Container cp      = frame.getContentPane();
      Action[] toolAct  = toolLoader.getActions();
      Action[] actions  = new Action[17 + toolAct.length];

      actions[0]  = new NewWindowAction();
      actions[1]  = new MenuEditorAction();
      actions[2]  = new CloseAction(fm,frame);
      actions[3]  = new ExitAction(fm);
      actions[4]  = new AssertAction(frame,resources,project);
      actions[5]  = new InsFileAction(frame,resources,project);
      actions[6]  = new CompileAction(frame,compiler);
      actions[7]  = new MakeAction(frame,compiler,linker);
      actions[8]  = new BuildAction(frame,compiler,linker);
      actions[9]  = new RunAction(frame,compiler,linker,execute);
      actions[10]  = new SelRunAction(frame,compiler,linker,execute);
      actions[11] = new ParamAction(frame);
      actions[12] = saveOnComAction;
      actions[13] = new CompilerOptionsAction(frame,compOptAction.isEnabled());
      actions[14] = new LinkerOptionsAction(frame,linkOptAction.isEnabled());
      actions[15] = new ExecuteOptionsAction(frame,execOptAction.isEnabled());
      actions[16] = new HelpOptionsAction(frame);
      for (int i=0; i<toolAct.length; i++) actions[17 + i] = toolAct[i];
      // create a codepad panel
      CodepadPanel codepad;

      if (loadAble) {
        codepad = new CodepadPanel(frame,resources,desktop,project,
                                   actions,file);
        if (lineNr > 0) codepad.goToLine(lineNr);
      }
      else {
        codepad = new CodepadPanel(frame,resources,desktop,project,actions);
      } // end if
      frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      cp.setLayout(new BorderLayout());
      cp.add("Center",codepad);
      frame.pack();
      frame.setSize(600,400);
      FrameUtil.shuffleLocation(frame);
      try {
        URL url = ResourceUtil.getResource(this,resources,FRAME_ICON);
        if (url != null) frame.setIconImage((new ImageIcon(url)).getImage());
      } catch (MissingResourceException mre) {
        // just don't use an icon
      }
      fm.register(frame,codepad);
      frame.show();
    } // end actionPerformed

    /** Returns a frame containing a codepad panel with the spec. file open. */
    protected Frame getFrameContaining(File file) {
      Enumeration enum = fm.getFrames();

      while (enum.hasMoreElements()) {
        JFrame parent    = (JFrame)enum.nextElement();
        Container c      = ((JFrame)parent).getContentPane();
        Component[] comp = c.getComponents();

        for (int i=0; i<comp.length; i++) {
          if (comp[i] instanceof CodepadPanel) {
            CodepadPanel cpp = (CodepadPanel)comp[i];

            if (file.equals(cpp.getCurrentFile())) return parent;
          } // end if
        } // end for
      } // end while
      return null;
    } // getFrameContaining

  } // end NewWindowAction

  /**
   * Constructs a new menu editor window.
   */
  class MenuEditorAction extends AbstractAction {

    // constructors
    MenuEditorAction() {
      super(MENUEDITOR_ACTION);
    } // end constructor MenuEditorAction

    // methods
    public void actionPerformed(ActionEvent e) {
      String title      = ResourceUtil.getResStringOrKey(resources,MEFRAME_TITLE);
      JFrame frame      = new JFrame(title);
      Container cp      = frame.getContentPane();
      Action[] toolAct  = toolLoader.getActions();
      Action[] actions  = new Action[4 + toolAct.length];

      actions[0]  = new NewWindowAction();
      actions[1]  = new MenuEditorAction();
      actions[2]  = new CloseAction(fm,frame);
      actions[3]  = new ExitAction(fm);
      for (int i=0; i<toolAct.length; i++) actions[4 + i] = toolAct[i];
      // create a menu editor panel
      MenuEditorPanel menuEditor;

      menuEditor = new MenuEditorPanel(frame,resources,desktop,project,actions);
      frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      cp.setLayout(new BorderLayout());
      cp.add("Center",menuEditor);
      frame.pack();
      frame.setSize(600,400);
      FrameUtil.shuffleLocation(frame);
      try {
        URL url = ResourceUtil.getResource(this,resources,MEFRAME_ICON);
        if (url != null) frame.setIconImage((new ImageIcon(url)).getImage());
      } catch (MissingResourceException mre) {
        // just don't use an icon
      }
      fm.register(frame,menuEditor);
      frame.show();
    } // end actionPerformed

  } // end MenuEditorAction

  class SplitWinAction extends AbstractAction {

    // constructors
    SplitWinAction() {
      super(SPLIT_ACTION);
    } // end constructor SplitWinAction

    // methods
    public void actionPerformed(ActionEvent e) {
      Boolean checked = Boolean.valueOf(desktop.getProperty(SPLIT_PROP));

      checked = checked.booleanValue() ? Boolean.FALSE : Boolean.TRUE;
      putValue("checked",checked);
      setFileManagerVisible(checked.booleanValue());
      desktop.put(SPLIT_PROP,checked.toString());
    } // end actionPerformed

  } // end SplitWinAction

  class AddAction extends AbstractAction {

    // constructors
    AddAction() {
      super(ADD_ACTION);
      setEnabled(false);
    } // end constructor AddAction

    // methods
    public void actionPerformed(ActionEvent e) {
      Object[] obj = fileManagerPanel.getSelectedValues();

      for (int i=0; i<obj.length; i++) {
        File f = (File)obj[i];

        if (f.isDirectory()) {
          if (!addDirectory(f)) break;
        }
        else {
          if (!projectFilePanel.addFile(f))
            Toolkit.getDefaultToolkit().beep();
          // end if
        } // end if
      } // end for
    } // end actionPerformed

  } // end AddAction

  /**
   * Sets a new project directory.
   */
  class SetDirAction extends AbstractAction {

    // constructors
    SetDirAction() {
      super(SETDIR_ACTION);
    } // end constructor SetDirAction

    // methods
    public void actionPerformed(ActionEvent e) {
      projectFilePanel.setHomeDirectory(fileManagerPanel.getPath());
    } // end actionPerformed

  } // end SetDirAction

  /** Display a dialog that lets you specify command line parameters. */
  class ParamAction extends AbstractAction {

    // instance variable declarations
    private Frame parentFrame;

    // constructors
    ParamAction() {
      super(PARAM_ACTION);
      this.parentFrame = null;
    } // end constructor ParamAction

    ParamAction(Frame parentFrame) {
      super(PARAM_ACTION);
      this.parentFrame = parentFrame;
    } // end constructor ParamAction

    // methods
    public void actionPerformed(ActionEvent e) {
      ParamDlg dlg;
      String str = project.getProperty(Const.CMD_LINE_PARAM);

      if (parentFrame == null) parentFrame = frame;
      dlg = new ParamDlg(parentFrame,resources,str);
      dlg.show();
    if (!dlg.isCanceled()) {
      str = project.getProperty(Const.CMD_LINE_PARAM);
      if (!dlg.getResult().equals(str)) {
        project.put(Const.CMD_LINE_PARAM,dlg.getResult());
        project.put(Const.PROJECT_CHANGED,"" + true);
      } // end if
    } // end if
    } // end actionPerformed

  } // end ParamAction

  /** Specify if editor file will be auto-saved before compile. */
  class SaveOnCompileAction extends AbstractAction {

    // constructors
    SaveOnCompileAction() {
      super(ASOC_ACTION);
    } // end constructor SaveOnCompileAction

    // methods
    public void actionPerformed(ActionEvent e) {
      Boolean checked = Boolean.valueOf(desktop.getProperty(Const.ASOC_PROP));

      checked = checked.booleanValue() ? Boolean.FALSE : Boolean.TRUE;
      putValue("checked",checked);
      desktop.put(Const.ASOC_PROP,checked.toString());
    } // end actionPerformed

  } // end SaveOnCompileAction

  /** Executes the compiler options dialog. */
  class CompilerOptionsAction extends AbstractAction {

    // instance variable declarations
    private Frame parentFrame;

    // constructors
    CompilerOptionsAction() {
      super(COMPILEROPTIONS_ACTION);
      parentFrame = null;
    } // end constructor CompilerOptionsAction

    CompilerOptionsAction(Frame parentFrame, boolean enbl) {
      super(COMPILEROPTIONS_ACTION);
      this.parentFrame = parentFrame;
      setEnabled(enbl);
    } // end constructor CompilerOptionsAction

    // methods
    public void actionPerformed(ActionEvent e) {
      if (parentFrame == null) parentFrame = frame;
      compiler.settings(parentFrame);
    } // end actionPerformed

  } // end CompilerOptionsAction

  /** Executes the linker options dialog. */
  class LinkerOptionsAction extends AbstractAction {

    // instance variable declarations
    private Frame parentFrame;

    // constructors
    LinkerOptionsAction() {
      super(LINKEROPTIONS_ACTION);
      parentFrame = null;
    } // end constructor LinkerOptionsAction

    LinkerOptionsAction(Frame parentFrame, boolean enbl) {
      super(LINKEROPTIONS_ACTION);
      this.parentFrame = parentFrame;
      setEnabled(enbl);
    } // end constructor LinkerOptionsAction

    // methods
    public void actionPerformed(ActionEvent e) {
      if (parentFrame == null) parentFrame = frame;
      linker.settings(parentFrame);
    } // end actionPerformed

  } // end LinkerOptionsAction

  /** Executes the execute options dialog. */
  class ExecuteOptionsAction extends AbstractAction {

    // instance variable declarations
    private Frame parentFrame;

    // constructors
    ExecuteOptionsAction() {
      super(EXECUTEOPTIONS_ACTION);
      parentFrame = null;
    } // end constructor ExecuteOptionsAction

    ExecuteOptionsAction(Frame parentFrame, boolean enbl) {
      super(EXECUTEOPTIONS_ACTION);
      this.parentFrame = parentFrame;
      setEnabled(enbl);
    } // end constructor ExecuteOptionsAction

    // methods
    public void actionPerformed(ActionEvent e) {
      if (parentFrame == null) parentFrame = frame;
      execute.settings(parentFrame);
    } // end actionPerformed

  } // end ExecuteOptionsAction

  /** Executes the help options dialog. */
  class HelpOptionsAction extends AbstractAction {

    // instance variable declarations
    private Frame parentFrame;

    // constructors
    HelpOptionsAction() {
      super(HELPOPTIONS_ACTION);
      parentFrame = null;
    } // end constructor HelpOptionsAction

    HelpOptionsAction(Frame parentFrame) {
      super(HELPOPTIONS_ACTION);
      this.parentFrame = parentFrame;
    } // end constructor HelpOptionsAction

    // methods
    public void actionPerformed(ActionEvent e) {
      String clazz = ResourceUtil.getResourceString(resources,Const.HELP_CLASS);
      AbstractHelpWindow wnd;

      if (parentFrame == null) parentFrame = frame;
      wnd = AbstractHelpWindow.getInstance(clazz,resources,desktop);
      if (wnd == null) setEnabled(false);
        else wnd.settings(parentFrame);
    } // end actionPerformed

  } // end HelpOptionsAction

  /** Starts the online help. */
  class ContentAction extends AbstractAction {

    // constructors
    ContentAction() {
      super(CONTENT_ACTION);
    } // end constructor ContentAction

    // instance methods
    public void actionPerformed(ActionEvent e) {
      String clazz = ResourceUtil.getResourceString(resources,Const.HELP_CLASS);
      AbstractHelpWindow wnd;

      wnd = AbstractHelpWindow.getInstance(clazz,resources,desktop);
      if (wnd == null) setEnabled(false);
        else wnd.showContext(HELP_TOPIC);
    } // end actionPerformed

  } // end ContentAction

  /** Display a application information. */
  class AboutAction extends AbstractAction {

    // constructors
    AboutAction() {
      super(ABOUT_ACTION);
    } // end constructor AboutAction

    // instance methods
    public void actionPerformed(ActionEvent e) {
      AboutDlg dlg = new AboutDlg(frame,resources,ABOUTDLG_PREFIX);

      dlg.show();
    } // end actionPerformed

  } // end AboutAction

} // end ProjectPanel

/* ******************************************************************
   end of file
*********************************************************************/
