// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import SID.Utils.*;

public abstract class ClassInfo extends DocumentedItem{

  protected static modifierPool modResolve;
  protected static int modMax;        // maximum number of modifiers

  public String superName;    // name of superclass
  public long lastModified;   // when was the source file modified
  public boolean valid;     // is the information valid (possible parser error)

  private BitSet modifiers;
  private Vector attributes;
  private Vector methods;

  public ClassInfo(){
    super();
    modifiers = new BitSet(10);
    attributes = new Vector(5,4);
    methods = new Vector(5,4);
    superName = "";
    valid = false;
  }

  public ClassInfo(String name, String sname){
    super(name);
    modifiers = new BitSet(10);
    attributes = new Vector(5,4);
    methods = new Vector(5,4);
    superName = sname;
    valid = false;
  }

  public void setModifier(int mod){
    modifiers.set(mod);
  }

  public void setModifiers(BitSet mods){
    modifiers = new BitSet();
    modifiers.or(mods);
  }

  public boolean isModifierSet(int mod){
    return modifiers.get(mod);
  }

  public boolean isModifierSet(String mod){
    return modifiers.get(modResolve.mapMod(mod));
  }

  public void removeModifier(int mod){
    modifiers.clear(mod);
  }

  public void addAttribute(AttributeInfo a){
    attributes.addElement(a);
  }

  public Enumeration attributes(){
    return attributes.elements();
  }

  public void addMethod(MethodInfo m){
    methods.addElement(m);
  }

  public Enumeration methods(){
    return methods.elements();
  }

  // java style output
  public String toLongString(){
    StringBuffer retString = new StringBuffer("");
    retString.append(modResolve.resolveModifiers(modifiers));
    retString.append("class " + name);
    if (superName.length() > 0) retString.append(" extends "+ superName);
    return retString.toString();
  }

  public abstract String getInterface();
  public abstract String getFilteredInterface(InterfaceFilter f);

  public void load(DataInputStream in) throws IOException{
    super.load(in);
    name = tool.readString(in);
    superName = tool.readString(in);

    lastModified = in.readLong();

    // write modifiers
    modifiers = tool.readBitSet(in);

    int size = in.readShort();
    AttributeInfo ai=null;
    Object o = null;

    attributes = new Vector(size);

    if (size > 0){
      o = tool.readClassInfo(in);
      for (int i=0; i<size; i++){
        try{
          ai = (AttributeInfo)(o.getClass().newInstance());
        }
        catch (InstantiationException e1) {e1.printStackTrace();}
        catch (IllegalAccessException e1) {e1.printStackTrace();}
        ai.load(in);
        attributes.addElement(ai);
      }
    }

    size = in.readShort();
    MethodInfo mi=null;
    methods = new Vector(size);

    if (size > 0){
      o = tool.readClassInfo(in);
      for (int i=0; i<size; i++){
        try{
          mi = (MethodInfo)(o.getClass().newInstance());
        }
        catch (InstantiationException e1) {System.out.println(e1.toString());}
        catch (IllegalAccessException e1) {System.out.println(e1.toString());}
        mi.load(in);
        methods.addElement(mi);
      }
    }
  }

  public void store(DataOutputStream out) throws IOException{
    super.store(out);
    tool.writeString(name, out);
    tool.writeString(superName, out);
    out.writeLong(lastModified);

    // write modifiers
    tool.writeBitSet(modifiers, modMax, out);
    out.writeShort(attributes.size());

    Enumeration e = attributes.elements();
    if (e.hasMoreElements()){
      AttributeInfo ai = (AttributeInfo)e.nextElement();
      tool.writeClassInfo(ai, out);
      ai.store(out);

      for (;e.hasMoreElements();){
        ai = (AttributeInfo)e.nextElement();
        ai.store(out);
      }
    }

    out.writeShort(methods.size());
//    System.out.println("storing " + methods.size()+" methods");
    e = methods.elements();
    if (e.hasMoreElements()){
      MethodInfo mi = (MethodInfo)e.nextElement();
      tool.writeClassInfo(mi, out);
      mi.store(out);

      for (;e.hasMoreElements();){
        mi = (MethodInfo)e.nextElement();
        mi.store(out);
      }
    }
  }
}