// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import SID.Utils.*;

// base class for method representation
// 
// no resolve of names is done, output in java style order
//
public class Parameter extends NamedItem{
	protected static modifierPool typeResolve;

	public short type;
	public short arraydim;					// 0 if simple type, n for n-dimensional arrays

	static{
		typeResolve = modifierPool.javaTypes();
	}

	public Parameter(){
		this("",(short)-1);
	}

	public Parameter(String name, short type){
		super(name);
		this.type = type;
		arraydim = 0;
	}

	public Parameter(String name, String type){
		super(name);
		this.type = (short)typeResolve.mapMod(type);
		arraydim = 0;
	}

	public String getType(){ return typeResolve.mapMod(type); }

	public boolean equals(Parameter other){
		return (name == other.name) && (type == other.type);
	}

	public String toLongString(){
		return typeResolve.mapMod(type) + " " + name;
	}

	public void load(DataInputStream in) throws IOException{
		super.load(in);
		name = in.readUTF();
		type = in.readShort();
		arraydim = in.readShort();
	}

	public void store(DataOutputStream out) throws IOException{
		super.store(out);
		out.writeUTF(name);
		out.writeShort(type);
		out.writeShort(arraydim);
	}
}
