/* ******************************************************************

   class    : AllowAllSecurityManager
   date     : 28.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Utils;

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.FileDescriptor;
import java.net.InetAddress;

/**
 * A security manager that allows access to all resources.  This class
 * is usefull if you want to implement a not very paranoid security
 * manager.
 *
 * @author puehringer edgar
 */
public class AllowAllSecurityManager extends SecurityManager {

  //constructors
  /**
   * Constructs a new SecurityManager. An application is not allowed to
   * create a new security manager if there is already a current security
   * manager.
   */
  protected AllowAllSecurityManager() {
    super();
  } // end constructor AllowAllSecurityManager

  //methods
  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * create a new class loader.
   */
  public void checkCreateClassLoader() {
    // do nothing
  } // end checkCreateClassLoader

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * modify the thread argument.
   */
  public void checkAccess(Thread g) {
    // do nothing
  } // end checkAccess

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * modify the thread group argument.
   */
  public void checkAccess(ThreadGroup g) {
    // do nothing
  } // end checkAccess

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * cause the Java Virtual Machine to halt with the specified status code.
   */
  public void checkExit(int status) {
    // do nothing
  } // end checkExit

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * create a subprocss.
   */
  public void checkExec(String cmd) {
    // do nothing
  } // end checkExec

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * dynamic link the library code specified by the string argument file.
   * The argument is either a simple library name or a complete filename.
   */
  public void checkLink(String lib) {
    // do nothing
  } // end checkLink

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * read from the specified file descriptor.
   */
  public void checkRead(FileDescriptor fd) {
    // do nothing
  } // end checkRead

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * read the file specified by the string argument.
   */
  public void checkRead(String file) {
    // do nothing
  } // end checkRead

  /**
   * Throws a SecurityException if the specified security context is not
   * allowed to read the file specified by the string argument.  The context
   * must be a security context returned by a previous call to
   * getSecurityContext.
   */
  public void checkRead(String file, Object context) {
    // do nothing
  } // end checkRead

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * write to the specified file descriptor.
   */
  public void checkWrite(FileDescriptor fd) {
    // do nothing
  } // end checkWrite

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * write to the file specified by the string argument.
   */
  public void checkWrite(String file) {
    // do nothing
  } // end checkWrite

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * delete the specified file.
   */
  public void checkDelete(String file) {
    // do nothing
  } // end checkDelete

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * open a socket connection to the specified host and port number.
   */
  public void checkConnect(String host, int port) {
    // do nothing
  } // end checkConnect

  /**
   * Throws a SecurityException if the specified security context is not
   * allowed to open a socket connection to the specified host and port
   * number.
   */
  public void checkConnect(String host, int port, Object context) {
    // do nothing
  } // end checkConnect

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * wait for a connection request on the specified local port number.
   */
  public void checkListen(int port) {
    // do nothing
  } // end checkListen

  /**
   * Throws a SecurityException if the calling thread is not permitted to
   * accept a socket connection from the specified host and port number.
   */
  public void checkAccept(String host, int port) {
    // do nothing
  } // end checkAccept

  /**
   * Tests if current execution context is allowed to use
   * (join/leave/send/receive) IP multicast.
   */
  public void checkMulticast(InetAddress maddr) {
    // do nothing
  } // end checkMulticast

  /**
   * Tests to see if current execution context is allowed to use
   * (join/leave/send/receive) IP multicast.
   */
  public void checkMulticast(InetAddress maddr, byte ttl) {
    // do nothing
  } // end checkMulticast

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * access or modify the system properties.
   */
  public void checkPropertiesAccess() {
    // do nothing
  } // end checkPropertiesAccess

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * access the system property with the specified key name.
   */
  public void checkPropertyAccess(String key) {
    // do nothing
  } // end checkPropertiesAccess

  /**
   * Returns false if the calling thread is not trusted to bring up the
   * top-level window indicated by the window argument.  In this case, the
   * caller can still decide to show the window, but the window should
   * include some sort of visual warning. If the method returns true, then
   * the window can be shown without any special restrictions.
   */
  public boolean checkTopLevelWindow(Object window) {
    return true;
  } // end checkTopLevelWindow

  /**
   * Tests if a client can initiate a print job request.
   */
  public void checkPrintJobAccess() {
    // do nothing
  } // end checkPrintJobAccess

  /**
   * Tests if a client can get access to the system clipboard.
   */
  public void checkSystemClipboardAccess() {
    // do nothing
  } // end checkSystemClipboardAccess

  /**
   * Tests if a client can get access to the AWT event queue.
   */
  public void checkAwtEventQueueAccess() {
    // do nothing
  } // end checkAwtEventQueueAccess

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * access the package specified by the argument.
   */
  public void checkPackageAccess(String pkg) {
    // do nothing
  } // end checkPackageAccess

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * define classes in the package specified by the argument.
   */
  public void checkPackageDefinition(String pkg) {
    // do nothing
  } // end checkPackageDefinition

  /**
   * Throws a SecurityException if the calling thread is not allowed to set
   * the socket factory used by ServerSocket or Socket, or the stream
   * handler factory used by URL.
   */
  public void checkSetFactory() {
    // do nothing
  } // end checkSetFactory

  /**
   * Tests if a client is allowed to access members. If access is denied,
   * throw a SecurityException. The default policy is to deny all accesses.
   */
  public void checkMemberAccess(Class clazz, int which) {
    // do nothing
  } // end checkMemberAccess

} // end AllowAllSecurityManager

/* ******************************************************************
   end of file
*********************************************************************/
