#define COMST "/*"
#define COMEN "*/"
#pragma unquote COMST
#pragma unquote COMEN
#pragma getstring PACKAGE Package
#pragma getbool APPLETENBL Applet enabled
COMST ******************************************************************

   class      : MenuLoader
#define __NAME__ MenuLoader
#define __EXT__ .java
   date       : __DATE__
#ifdef __COMPILER__
   compiler : __COMPILER__
#endif

******************************************************************* COMEN

#ifdef PACKAGE
package PACKAGE;
  #define __RELPATH__ PACKAGE

#endif
/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Builds up a AWT menu by reading the settings from a resource file.  It's
 * intended to use this class for loading menu's created with Magic Menu.
 * The class can create Panels that look and behave like menubars with the
 * method <code>createMenuPanel</code>. This menu panels can be used to
 * realize menubars in applets for browsers supporting Java 1.1. Open the
 * project <code>HelloW.prj</code> in the subdirectory <code>example</code>
 * and run <code>HelloW.html</code> to see a demonstration of this.
 *
 * @author Puehringer Edgar
 */
public class MenuLoader {

  // class variable declarations
  /**
   * Suffix applied to the key used in resource file
   * lookups for a label.
   */
  public static final String LABEL_SUFFIX = "Label";

  /**
   * Suffix applied to the key used in resource file
   * lookups for a accelerator.
   */
  public static final String ACCEL_SUFFIX = "Accel";

  /**
   * Suffix applied to the key used in resource file
   * lookups for an action.
   */
  public static final String ACTION_SUFFIX = "Action";

  /**
   * Suffix applied to the key used in resource file
   * lookups for the type of a menu item.
   */
  public static final String TYPE_SUFFIX = "Type";

  /** Type identifier of a menu item. */
  public static final String TYPE_ITEM = "menuitem";

  /** Type identifier of a submenu. */
  public static final String TYPE_MENU = "menu";

  /** Type identifier of a checkbox menu item. */
  public static final String TYPE_CHECKBOX = "checkbox";

  /** The escape character to identify a mnemonic in a label. */
  public static final char MNEMONIC_ESCAPE = '&';

  /** A reference to the java.awt.event.InputEvent class. */
  private static Class eventClass;
  /** A reference to the java.awt.event.KeyEvent class. */
  private static Class keyEventClass;

  // instance variable declarations
  private ResourceBundle resources;
  private Properties properties;
  private ActionListener defaultAction;

  // static initializers
  static {
    try {
      keyEventClass = Class.forName("java.awt.event.KeyEvent");
    } catch (ClassNotFoundException e) { e.printStackTrace(); }
    try {
      eventClass = Class.forName("java.awt.Event");
    } catch (ClassNotFoundException e) { e.printStackTrace(); }
  } // end static initializer

  // constructors
  /**
   * Create a new JMenuLoader object.
   *
   * @param resources A resources bundle containing the menu settings.
   */
  public MenuLoader(ResourceBundle resources) {
    this.resources = resources;
    properties     = null;
    defaultAction  = null;
  } // end constructor MenuLoader

  /**
   * Create a new JMenuLoader object.
   *
   * @param properties Properties containing the menu settings.
   */
  public MenuLoader(Properties properties) {
    resources       = null;
    this.properties = properties;
    defaultAction   = null;
  } // end constructor MenuLoader

  // class methods
  /** Filters the mnemonic identifiers out of a label string. */
  private static String filterMnemonic(String label, char[] ch) {
    int index = label.indexOf(MNEMONIC_ESCAPE);

    if (index < 0) return label;
    if ((index + 1) < label.length()) {
      char c = label.charAt(index + 1);

      if (c != MNEMONIC_ESCAPE) ch[0] = c;
      return label.substring(0,index) + c +
        filterMnemonic(label.substring(index + 2),ch);
    }
    else {
      return label.substring(0,index);
    } // end if
  } // end filterMnemonic

  // methods
  /**
   * Returns the default action listener.  This ActionListener is
   * registered at every menu item.
   */
  public ActionListener getDefaultActionListener() {
    return defaultAction;
  } // end getDefaultActionListener

  /**
   * Set the default action listener.  This action listener is registered at
   * every menu item. If a ActionListener is set, this should be done
   * before the first call to createMenuBar, createMenu or createMenuItem.
   *
   * @see #createMenuBar
   * @see #createMenu
   * @see #createMenuItem
   */
  public void setDefaultActionListener(ActionListener al) {
    this.defaultAction = al;
  } // end setDefaultActionListener

  /**
   * This is the hook through which all menu items are created.
   * CheckbBoxMenuItems are connected by a special adapter class, which
   * ensures compatibility to the ActionListener interface.
   */
  public MenuItem createMenuItem(String key) {
    String label = getResourceString(key + LABEL_SUFFIX);
    String type  = getResourceString(key + TYPE_SUFFIX);
    String astr  = getResourceString(key + ACTION_SUFFIX);
    String acc   = getResourceString(key + ACCEL_SUFFIX);
    MenuItem mi;
    ActionListener a;
    boolean isSubmenu;
    ActionAdapter adapter;
    char[] mnemonic = new char[1];

    // prepare the menu item label
    if (label == null) {
      reportError("Label for menu item key \"" + key + "\" not found");
      return null;
    }
    else {
      label = filterMnemonic(label,mnemonic);
    } // end if
    // create a menu item matching the spezified type
    if (type != null) type = type.toLowerCase();
    if (astr == null) astr = key;
    if ((type == null) || type.equals(TYPE_ITEM)) {
      mi = new MenuItem(label);
      mi.setActionCommand(astr);
      a = getAction(astr);
      isSubmenu = false;
      adapter   = null;
    } else if (type.equals(TYPE_CHECKBOX)) {
      CheckboxMenuItem cbmi = new CheckboxMenuItem(label);

      cbmi.setActionCommand(astr);
      a = getAction(astr);
      isSubmenu = false;
      adapter   = new ActionAdapter();
      cbmi.addItemListener(adapter);
      mi = cbmi;
    } else if (type.equals(TYPE_MENU)) {
      mi        = createMenu(key);
      a         = null;
      isSubmenu = true;
      adapter   = null;
    } else {
      reportError("Unknown menu type: " + type);
      return null;
    } // end if
    // connect an action to the menu command
    if (a != null) {
      if (adapter == null) mi.addActionListener(a);
        else adapter.addActionListener(a);
    } else if (!isSubmenu) {
      mi.setEnabled(false);
    } // end if
    // set the accelerator
    if (acc != null) {
      try {
        String[] arr = tokenize(acc);
        int value    = keyEventClass.getField(arr[0]).getInt(null);
        int mod      = Event.CTRL_MASK;

        for (int i=1; i<arr.length; i++) {
          mod ^= eventClass.getField(arr[i]).getInt(null);
        } // end for
        mi.setShortcut(new MenuShortcut(value,
          ((mod & Event.SHIFT_MASK) != 0)));
      } catch (NoSuchFieldException e1) {
        reportError("Unknown keycode: " + acc);
      } catch (SecurityException e2) {
        e2.printStackTrace();
      } catch (IllegalAccessException e3) {
        e3.printStackTrace();
      }
    } // end if
    return mi;
  } // end createMenuItem

  /**
   * Return the default action listener.  This Method should be overwritten,
   * if not all menu items should register the same action listener.
   *
   * @param cmd The action command of the associated menu item.
   */
  protected ActionListener getAction(String cmd) {
    return defaultAction;
  } // end getAction

  /**
   * Read a string from the resource bundle containing the menu
   * definition.
   */
  protected String getResourceString(String key) {
    String str;

    if (resources != null) {
      try {
        str = resources.getString(key);
      } catch (MissingResourceException mre) {
        str = null;
      }
    }
    else {
      str = properties.getProperty(key);
    } // end if
    return str;
  } // end getResourceString

  /**
   * Take the given string and chop it up into a series
   * of strings on whitespace boundries.  This is useful
   * for trying to get an array of strings out of the
   * resource file.
   */
  protected String[] tokenize(String input) {
    Vector v          = new Vector();
    StringTokenizer t = new StringTokenizer(input);
    String cmd[];

    while (t.hasMoreTokens()) v.addElement(t.nextToken());
    cmd = new String[v.size()];
    for (int i=0; i<cmd.length; i++) cmd[i] = (String)v.elementAt(i);
    return cmd;
  } // end tokenize

  /**
   * Create the menubar for the specified key.  This pulls the
   * definition of the menu from the associated resource file.
   */
  public MenuBar createMenuBar(String key) {
    MenuBar mb   = new MenuBar();
    String mbDesc = getResourceString(key);
    String[] menuKeys;

    if (mbDesc == null) {
      reportError("MenuBar \"" + key + "\" not found or empty");
      return null;
    } // end if
    menuKeys = tokenize(mbDesc);
    for (int i=0; i<menuKeys.length; i++) {
      Menu m = createMenu(menuKeys[i]);

      if (m != null) mb.add(m);
    } // end for
    return mb;
  } // end createMenuBar

#ifdef APPLETENBL
  /**
   * Create a panel of popup menu labels for the specified key.
   * This pulls the definition of the menu from the associated
   * resource file.  Only available, if 'Applet enabled' is
   * checked.
   */
  public Panel createMenuPanel(String key) {
    Panel p       = new Panel();
    String mbDesc = getResourceString(key);
    String[] menuKeys;
    boolean[] pressed;

    pressed    = new boolean[1];
    pressed[0] = false;
    if (mbDesc == null) {
      reportError("MenuBar \"" + key + "\" not found or empty");
      return null;
    } // end if
    menuKeys = tokenize(mbDesc);
    for (int i=0; i<menuKeys.length; i++) {
      PopupLabel pl = createPopupLabel(menuKeys[i],pressed);

      if (pl != null) p.add(pl);
    } // end for
    return p;
  } // end createMenuPanel

#endif
  /**
   * Create a menu.  This pulls the definition
   * of the menu from the associated resource file.
   */
  protected Menu createMenu(String key) {
    String label    = getResourceString(key + LABEL_SUFFIX);
    String menuDesc = getResourceString(key);
    String[] itemKeys;
    Menu menu;
    char[] mnemonic = new char[1];

    if (menuDesc == null) {
      reportError("Menu \"" + key + "\" not found or empty");
      return null;
    } // end if
    itemKeys = tokenize(menuDesc);
    // prepare the menu label
    if (label == null) {
      reportError("Label for menu key \"" + key + "\" not found");
      return null;
    } // end if
    label = filterMnemonic(label,mnemonic);
    if (menuDesc == null) {
      reportError("Menu \"" + key + "\" not found or empty");
      return null;
    } // end if
    menu = new Menu(label);
    for (int i = 0; i < itemKeys.length; i++) {
      if (itemKeys[i].equals("-")) {
        menu.addSeparator();
      }
      else {
        MenuItem mi = createMenuItem(itemKeys[i]);

        if (mi != null) menu.add(mi);
      } // end if
    } // end for
    return menu;
  } // end createMenu

#ifdef APPLETENBL
  /**
   * Create a label with popup menu.  The definition of the
   * popup menu is pulled from the associated resource file.
   * Only available, if 'Applet enabled' is checked.
   */
  protected PopupLabel createPopupLabel(String key, boolean[] pressed) {
    String label    = getResourceString(key + LABEL_SUFFIX);
    String menuDesc = getResourceString(key);
    String[] itemKeys;
    PopupMenu menu;
    char[] mnemonic = new char[1];

    if (menuDesc == null) {
      reportError("Menu \"" + key + "\" not found or empty");
      return null;
    } // end if
    itemKeys = tokenize(menuDesc);
    // prepare the menu label
    if (label == null) {
      reportError("Label for menu key \"" + key + "\" not found");
      return null;
    } // end if
    label = filterMnemonic(label,mnemonic);
    if (menuDesc == null) {
      reportError("Menu \"" + key + "\" not found or empty");
      return null;
    } // end if
    menu = new PopupMenu(label);
    for (int i = 0; i < itemKeys.length; i++) {
      if (itemKeys[i].equals("-")) {
        menu.addSeparator();
      }
      else {
        MenuItem mi = createMenuItem(itemKeys[i]);

        if (mi != null) {
          mi.deleteShortcut();
          menu.add(mi);
        } // end if
      } // end if
    } // end for
    return new PopupLabel(label,menu,pressed);
  } // end createPopupLabel

#endif
  /**
   * This method will be called, if some errors occur.  By default this
   * will cause an output to stderr.
   */
  protected void reportError(String text) {
    System.err.println("MenuLoader: " + text + ".");
  } // end reportError

  // inner classes
  /**
   * Converts an item event to an action event.  This is useful
   * when dealing around with CheckboxMenuItems.
   */
  class ActionAdapter implements ItemListener {

    // instance variable declarations
    Hashtable listeners;

    // constructors
    ActionAdapter() {
      listeners = new Hashtable(1);
    } // end constructor ActionAdapter

    // methods
    /**
     * Adds the specified action listener to receive action events.
     *
     * @param l The action listener.
     */
    public synchronized void addActionListener(ActionListener l) {
      listeners.put(l,new Object());
    } // end addActionListener

    /**
     * Removes the specified action listener so that it no longer
     * receives action events.
     *
     * @param l The action listener.
     */
    public synchronized void removeActionListener(ActionListener l) {
      listeners.remove(l);
    } // end removeActionListener

    // --- ItemListener -----------------------------------------------
    /** Invoked when an item's state has been changed. */
    public void itemStateChanged(ItemEvent e) {
      Enumeration enum = listeners.keys();
      Object obj       = e.getSource();
      String cmd       = obj instanceof CheckboxMenuItem ?
                         ((CheckboxMenuItem)obj).getActionCommand() :
                         "";

      while (enum.hasMoreElements()) {
        ActionListener al = (ActionListener)enum.nextElement();
        ActionEvent ae;

        ae = new ActionEvent(obj,ActionEvent.ACTION_PERFORMED,cmd);
        al.actionPerformed(ae);
      } // end while
    } // end itemStateChanged

  } // end ActionAdapter

#ifdef APPLETENBL
  /**
   * A label that looks and behaves like a menu.
   * Only available, if 'Applet enabled' is checked.
   */
  class PopupLabel extends Label {

    // instance variable declarations
    protected PopupMenu popup;
    /** True, if the mouse had been pressed in a PopupLabel. */
    protected boolean[] pressed;
    /** True if the mouse pointer entered and pressed is true. */
    protected boolean entered;

    // constructors
    public PopupLabel(String text, PopupMenu popup, boolean[] pressed) {
      super(text,Label.CENTER);
      this.popup = popup;
      add(this.popup);
      enableEvents(AWTEvent.MOUSE_EVENT_MASK |
                   AWTEvent.MOUSE_MOTION_EVENT_MASK);
      this.pressed = pressed;
      entered      = false;
    } // end constructor PopupLabel

    // methods
    public void processMouseEvent(MouseEvent e) {
      Component comp;

      switch (e.getID()) {
        case MouseEvent.MOUSE_PRESSED:
          comp = e.getComponent();
          comp.setBackground(SystemColor.textHighlight);
          comp.setForeground(SystemColor.textHighlightText);
          pressed[0] = true;
        break;
        case MouseEvent.MOUSE_RELEASED:
          if (pressed[0]) {
            comp = e.getComponent();
            comp.setBackground(SystemColor.control);
            comp.setForeground(SystemColor.controlText);
            pressed[0] = false;
            popup.show(comp,0,comp.getSize().height - 1);
          } // end if
        break;
        case MouseEvent.MOUSE_ENTERED:
          if (pressed[0]) entered = true;
        break;
        case MouseEvent.MOUSE_EXITED:
          if (pressed[0]) {
            comp = e.getComponent();
            comp.setBackground(SystemColor.control);
            comp.setForeground(SystemColor.controlText);
          } // end if
          entered = false;
        break;
        default:
      } // end switch
      super.processMouseEvent(e);
    } // end processMouseEvent

    public void processMouseMotionEvent(MouseEvent e) {
      Component comp;

      if ((e.getID() == MouseEvent.MOUSE_DRAGGED) && entered) {
        comp = e.getComponent();
        comp.setBackground(SystemColor.textHighlight);
        comp.setForeground(SystemColor.textHighlightText);
        entered = false;
      } // end if
      super.processMouseMotionEvent(e);
    } // end processMouseMotionEvent

  } // end PopupLabel

#endif
} // end MenuLoader

/* ******************************************************************
   end of file
*********************************************************************/
